"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PageHostedApiBlockWrapper = exports._implListRelatedItemsByRelType = exports._implListRelatedBlocksByRelType = exports._implIterRelatedItemsByRelType = exports._implIterRelatedBlocksByRelType = exports.setUnion = exports.setIntersection = exports.normalizeOptionalSet = exports.argMax = exports.aggregate = exports.modalAvg = exports.AggregationMethod = exports.indent = exports.escapeHtml = exports.getIterable = exports.DocumentMetadata = exports.ApiBlockWrapper = exports.doesFilterAllowBlockType = exports.ApiObjectWrapper = void 0;
class ApiObjectWrapper {
    constructor(dict) {
        this._dict = dict;
    }
    get dict() {
        return this._dict;
    }
}
exports.ApiObjectWrapper = ApiObjectWrapper;
function doesFilterAllowBlockType(filterOpts, blockType) {
    if (!filterOpts)
        return true;
    if (filterOpts.skipBlockTypes) {
        const skipBlockTypes = normalizeOptionalSet(filterOpts.skipBlockTypes);
        if (skipBlockTypes.has(blockType))
            return false;
    }
    if (filterOpts.includeBlockTypes) {
        const includeBlockTypes = normalizeOptionalSet(filterOpts.includeBlockTypes);
        if (!includeBlockTypes.has(blockType))
            return false;
    }
    return true;
}
exports.doesFilterAllowBlockType = doesFilterAllowBlockType;
class ApiBlockWrapper extends ApiObjectWrapper {
    get id() {
        return this._dict.Id;
    }
    get blockType() {
        return this._dict.BlockType;
    }
    get childBlockIds() {
        return this.relatedBlockIdsByRelType("CHILD");
    }
    relatedBlockIdsByRelType(relType) {
        const isMultiType = Array.isArray(relType);
        let ids = [];
        (this._dict.Relationships || []).forEach((rs) => {
            if (isMultiType) {
                if (relType.indexOf(rs.Type) >= 0) {
                    ids = ids.concat(rs.Ids);
                }
            }
            else {
                if (rs.Type === relType) {
                    ids = ids.concat(rs.Ids);
                }
            }
        });
        return ids;
    }
}
exports.ApiBlockWrapper = ApiBlockWrapper;
class DocumentMetadata extends ApiObjectWrapper {
    get nPages() {
        var _a;
        return ((_a = this._dict) === null || _a === void 0 ? void 0 : _a.Pages) || 0;
    }
}
exports.DocumentMetadata = DocumentMetadata;
function getIterable(collectionFetcher) {
    const getIterator = () => {
        const collection = collectionFetcher();
        let ixItem = 0;
        return {
            next: () => {
                return ixItem < collection.length
                    ? {
                        done: false,
                        value: collection[ixItem++],
                    }
                    : {
                        done: true,
                        value: undefined,
                    };
            },
        };
    };
    return {
        [Symbol.iterator]: getIterator,
    };
}
exports.getIterable = getIterable;
function escapeHtml(str, { forAttr = false } = {}) {
    return str.replace(forAttr ? /[&<>'"]/g : /[&<>]/g, (entity) => ({
        "&": "&amp;",
        "<": "&lt;",
        ">": "&gt;",
        "'": "&#39;",
        '"': "&quot;",
    })[entity]);
}
exports.escapeHtml = escapeHtml;
function indent(text, { character = "\t", count = 1, includeEmptyLines = false, skipFirstLine = false } = {}) {
    const result = text.replace(includeEmptyLines ? /^/gm : /^(?!\s*$)/gm, character.repeat(count));
    if (skipFirstLine) {
        return result.substring(count * character.length);
    }
    else {
        return result;
    }
}
exports.indent = indent;
var AggregationMethod;
(function (AggregationMethod) {
    AggregationMethod["GeometricMean"] = "GEOMEAN";
    AggregationMethod["Max"] = "MAX";
    AggregationMethod["Mean"] = "MEAN";
    AggregationMethod["Min"] = "MIN";
    AggregationMethod["Mode"] = "MODE";
})(AggregationMethod || (exports.AggregationMethod = AggregationMethod = {}));
function modalAvg(arr) {
    const freqs = {};
    for (const item of arr) {
        if (freqs[item]) {
            ++freqs[item].freq;
        }
        else {
            freqs[item] = { value: item, freq: 1 };
        }
    }
    let maxFreq = 0;
    let mode = null;
    for (const item in freqs) {
        if (freqs[item].freq > maxFreq) {
            maxFreq = freqs[item].freq;
            mode = freqs[item].value;
        }
    }
    return mode;
}
exports.modalAvg = modalAvg;
function aggregate(arr, aggMethod) {
    const actualArr = Array.isArray(arr) ? arr : Array.from(arr);
    if (actualArr.length === 0)
        return null;
    if (aggMethod === "GEOMEAN") {
        return Math.exp(actualArr.reduce((acc, next) => acc + Math.log(next), 0) / actualArr.length);
    }
    else if (aggMethod === "MAX") {
        return Math.max(...actualArr);
    }
    else if (aggMethod === "MEAN") {
        return actualArr.reduce((acc, next) => acc + next, 0) / actualArr.length;
    }
    else if (aggMethod === "MIN") {
        return Math.min(...actualArr);
    }
    else if (aggMethod === "MODE") {
        return modalAvg(actualArr);
    }
    else {
        throw new Error(`Unsupported aggMethod '${aggMethod}' not in allowed AggregationMethod enum`);
    }
}
exports.aggregate = aggregate;
function argMax(arr) {
    return arr.reduce((state, nextVal, nextIx) => (nextVal > state.maxValue ? { maxValue: nextVal, maxIndex: nextIx } : state), { maxValue: -Infinity, maxIndex: -1 });
}
exports.argMax = argMax;
function normalizeOptionalSet(raw) {
    if (raw instanceof Set)
        return raw;
    if (raw === null || typeof raw === "undefined")
        return null;
    if (Array.isArray(raw))
        return new Set(raw);
    return new Set([raw]);
}
exports.normalizeOptionalSet = normalizeOptionalSet;
function setIntersection(a, b) {
    const result = new Set(a);
    for (const entry of result) {
        if (!b.has(entry))
            result.delete(entry);
    }
    return result;
}
exports.setIntersection = setIntersection;
function setUnion(a, b) {
    const result = new Set(a);
    for (const entry of b) {
        result.add(entry);
    }
    return result;
}
exports.setUnion = setUnion;
function _implIterRelatedBlocksByRelType(relType, { includeBlockTypes = null, onMissingBlockId = "error", onUnexpectedBlockType = null, skipBlockTypes = null, } = {}, me, host) {
    includeBlockTypes = normalizeOptionalSet(includeBlockTypes);
    skipBlockTypes = normalizeOptionalSet(skipBlockTypes);
    const getIterator = () => {
        const blockIds = me.relatedBlockIdsByRelType(relType);
        let ixItem = 0;
        return {
            next: () => {
                while (ixItem < blockIds.length) {
                    const blockId = blockIds[ixItem++];
                    const block = host.getBlockById(blockId);
                    if (!block) {
                        if (!onMissingBlockId)
                            continue;
                        const msg = `(While iterating ${relType} relations of parent ${me.id}) Referenced block ID ${blockId} not found!`;
                        if (onMissingBlockId === "warn") {
                            console.warn(msg);
                            continue;
                        }
                        else {
                            throw new Error(msg);
                        }
                    }
                    if (skipBlockTypes && skipBlockTypes.has(block.BlockType))
                        continue;
                    if (includeBlockTypes && !includeBlockTypes.has(block.BlockType)) {
                        if (!onUnexpectedBlockType)
                            continue;
                        const msg = `(While iterating ${relType} relations of parent ${me.id}) Found unexpected block ID ${blockId} of type ${block.BlockType} not in set ${Array.from(includeBlockTypes)}`;
                        if (onUnexpectedBlockType === "warn") {
                            console.warn(msg);
                            continue;
                        }
                        else {
                            throw new Error(msg);
                        }
                    }
                    return {
                        done: false,
                        value: block,
                    };
                }
                return {
                    done: true,
                    value: undefined,
                };
            },
        };
    };
    return {
        [Symbol.iterator]: getIterator,
    };
}
exports._implIterRelatedBlocksByRelType = _implIterRelatedBlocksByRelType;
function _implIterRelatedItemsByRelType(relType, opts = {}, me, host) {
    const baseIterable = _implIterRelatedBlocksByRelType(relType, opts, me, host);
    const getIterator = () => {
        const baseIterator = baseIterable[Symbol.iterator]();
        return {
            next: () => {
                const nextResult = baseIterator.next();
                if (nextResult.done)
                    return nextResult;
                return {
                    done: false,
                    value: host.getItemByBlockId(nextResult.value.Id, nextResult.value.BlockType),
                };
            },
        };
    };
    return {
        [Symbol.iterator]: getIterator,
    };
}
exports._implIterRelatedItemsByRelType = _implIterRelatedItemsByRelType;
function _implListRelatedBlocksByRelType(relType, { includeBlockTypes = null, onMissingBlockId = "error", onUnexpectedBlockType = null, skipBlockTypes = null, } = {}, me, host) {
    includeBlockTypes = normalizeOptionalSet(includeBlockTypes);
    skipBlockTypes = normalizeOptionalSet(skipBlockTypes);
    let blocks = [];
    for (const blockId of me.relatedBlockIdsByRelType(relType)) {
        const block = host.getBlockById(blockId);
        if (!block) {
            if (!onMissingBlockId)
                continue;
            const msg = `(While iterating ${relType} relations of parent ${me.id}) Referenced block ID ${blockId} not found!`;
            if (onMissingBlockId === "warn") {
                console.warn(msg);
                continue;
            }
            else {
                throw new Error(msg);
            }
        }
        blocks.push(block);
    }
    if (skipBlockTypes)
        blocks = blocks.filter((block) => !skipBlockTypes.has(block.BlockType));
    if (includeBlockTypes) {
        blocks = blocks.filter((block) => {
            if (includeBlockTypes.has(block.BlockType))
                return true;
            if (!onUnexpectedBlockType)
                return false;
            const msg = `(While iterating ${relType} relations of parent ${me.id}) Found unexpected block ID ${block.Id} of type ${block.BlockType} not in set ${Array.from(includeBlockTypes)}`;
            if (onUnexpectedBlockType === "warn") {
                console.warn(msg);
                return false;
            }
            else {
                throw new Error(msg);
            }
        });
    }
    return blocks;
}
exports._implListRelatedBlocksByRelType = _implListRelatedBlocksByRelType;
function _implListRelatedItemsByRelType(relType, opts = {}, me, host) {
    return _implListRelatedBlocksByRelType(relType, opts, me, host).map((block) => host.getItemByBlockId(block.Id, block.BlockType));
}
exports._implListRelatedItemsByRelType = _implListRelatedItemsByRelType;
class PageHostedApiBlockWrapper extends ApiBlockWrapper {
    constructor(dict, parentPage) {
        super(dict);
        this._parentPage = parentPage;
        parentPage.registerParsedItem(dict.Id, this);
    }
    get parentPage() {
        return this._parentPage;
    }
    iterRelatedItemsByRelType(relType, opts = {}) {
        return _implIterRelatedItemsByRelType(relType, opts, this, this.parentPage);
    }
    listRelatedItemsByRelType(relType, opts = {}) {
        return _implListRelatedItemsByRelType(relType, opts, this, this.parentPage);
    }
}
exports.PageHostedApiBlockWrapper = PageHostedApiBlockWrapper;
//# sourceMappingURL=base.js.map