"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Signature = exports.SelectionElement = exports.LineGeneric = exports.WithWords = exports.buildWithContent = exports.Word = void 0;
const base_1 = require("./base");
const geometry_1 = require("./geometry");
class Word extends base_1.ApiBlockWrapper {
    constructor(block) {
        super(block);
        this._geometry = new geometry_1.Geometry(block.Geometry, this);
    }
    get confidence() {
        return this._dict.Confidence;
    }
    set confidence(newVal) {
        this._dict.Confidence = newVal;
    }
    get geometry() {
        return this._geometry;
    }
    get text() {
        return this._dict.Text;
    }
    get textType() {
        return this._dict.TextType;
    }
    set textType(newVal) {
        this._dict.TextType = newVal;
    }
    html(opts) {
        if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        return (0, base_1.escapeHtml)(this.text);
    }
    str() {
        return this.text;
    }
}
exports.Word = Word;
function buildWithContent({ contentTypes = ["SELECTION_ELEMENT", "SIGNATURE", "WORD"], onUnexpectedBlockType = null, otherExpectedChildTypes = null, } = {}) {
    const contentTypesSet = new Set(contentTypes);
    const defaultOnUnexpected = onUnexpectedBlockType;
    const otherTypesSet = otherExpectedChildTypes
        ? new Set(otherExpectedChildTypes)
        : new Set();
    return function WithContent(SuperClass) {
        return class extends SuperClass {
            getText(opts) {
                if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
                    return "";
                return this.listContent(opts)
                    .map((c) => c.text)
                    .join(" ");
            }
            iterContent({ includeBlockTypes = null, onUnexpectedBlockType = defaultOnUnexpected, skipBlockTypes = null, } = {}) {
                if (includeBlockTypes) {
                    includeBlockTypes = (0, base_1.normalizeOptionalSet)(includeBlockTypes);
                    includeBlockTypes = (0, base_1.setIntersection)(contentTypesSet, includeBlockTypes);
                }
                else {
                    includeBlockTypes = contentTypesSet;
                }
                if (skipBlockTypes) {
                    skipBlockTypes;
                    skipBlockTypes = (0, base_1.normalizeOptionalSet)(skipBlockTypes);
                    skipBlockTypes = (0, base_1.setUnion)(skipBlockTypes, otherTypesSet);
                }
                else {
                    skipBlockTypes = otherTypesSet;
                }
                return this.iterRelatedItemsByRelType("CHILD", {
                    includeBlockTypes,
                    onUnexpectedBlockType,
                    skipBlockTypes,
                });
            }
            listContent({ includeBlockTypes = null, onUnexpectedBlockType = defaultOnUnexpected, skipBlockTypes = null, } = {}) {
                if (includeBlockTypes) {
                    includeBlockTypes = (0, base_1.normalizeOptionalSet)(includeBlockTypes);
                    includeBlockTypes = (0, base_1.setIntersection)(contentTypesSet, includeBlockTypes);
                }
                else {
                    includeBlockTypes = contentTypesSet;
                }
                if (skipBlockTypes) {
                    skipBlockTypes;
                    skipBlockTypes = (0, base_1.normalizeOptionalSet)(skipBlockTypes);
                    skipBlockTypes = (0, base_1.setUnion)(skipBlockTypes, otherTypesSet);
                }
                else {
                    skipBlockTypes = otherTypesSet;
                }
                return this.listRelatedItemsByRelType("CHILD", {
                    includeBlockTypes,
                    onUnexpectedBlockType,
                    skipBlockTypes,
                });
            }
            get nContentItems() {
                return this.listContent().length;
            }
            get text() {
                return this.getText();
            }
        };
    };
}
exports.buildWithContent = buildWithContent;
function WithWords(SuperClass, { onUnexpectedBlockType = null, otherExpectedChildTypes = null } = {}) {
    const contentTypesSet = new Set(["WORD"]);
    const defaultOnUnexpected = onUnexpectedBlockType;
    const otherTypesSet = otherExpectedChildTypes
        ? new Set(otherExpectedChildTypes)
        : new Set();
    return class extends SuperClass {
        getText(opts) {
            if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
                return "";
            return this.listWords(opts)
                .map((w) => w.text)
                .join(" ");
        }
        iterWords({ includeBlockTypes = null, onUnexpectedBlockType = defaultOnUnexpected, skipBlockTypes = null, } = {}) {
            if (includeBlockTypes) {
                includeBlockTypes = (0, base_1.normalizeOptionalSet)(includeBlockTypes);
                includeBlockTypes = (0, base_1.setIntersection)(contentTypesSet, includeBlockTypes);
            }
            else {
                includeBlockTypes = contentTypesSet;
            }
            if (skipBlockTypes) {
                skipBlockTypes;
                skipBlockTypes = (0, base_1.normalizeOptionalSet)(skipBlockTypes);
                skipBlockTypes = (0, base_1.setUnion)(skipBlockTypes, otherTypesSet);
            }
            else {
                skipBlockTypes = otherTypesSet;
            }
            return this.iterRelatedItemsByRelType("CHILD", {
                includeBlockTypes,
                onUnexpectedBlockType,
                skipBlockTypes,
            });
        }
        listWords({ includeBlockTypes = null, onUnexpectedBlockType = defaultOnUnexpected, skipBlockTypes = null, } = {}) {
            if (includeBlockTypes) {
                includeBlockTypes = (0, base_1.normalizeOptionalSet)(includeBlockTypes);
                includeBlockTypes = (0, base_1.setIntersection)(contentTypesSet, includeBlockTypes);
            }
            else {
                includeBlockTypes = contentTypesSet;
            }
            if (skipBlockTypes) {
                skipBlockTypes;
                skipBlockTypes = (0, base_1.normalizeOptionalSet)(skipBlockTypes);
                skipBlockTypes = (0, base_1.setUnion)(skipBlockTypes, otherTypesSet);
            }
            else {
                skipBlockTypes = otherTypesSet;
            }
            return this.listRelatedItemsByRelType("CHILD", {
                includeBlockTypes,
                onUnexpectedBlockType,
                skipBlockTypes,
            });
        }
        get nWords() {
            return this.listWords().length;
        }
        get text() {
            return this.getText();
        }
        wordAtIndex(ix) {
            if (ix < 0)
                throw new Error(`Word index ${ix} must be >=0`);
            let ixCurr = 0;
            for (const word of this.iterWords()) {
                if (ixCurr === ix)
                    return word;
                ++ixCurr;
            }
            throw new Error(`Word index ${ix} out of bounds for length ${ixCurr}`);
        }
    };
}
exports.WithWords = WithWords;
class LineGeneric extends WithWords(base_1.PageHostedApiBlockWrapper) {
    constructor(block, parentPage) {
        super(block, parentPage);
        this._geometry = new geometry_1.Geometry(block.Geometry, this);
    }
    get confidence() {
        return this._dict.Confidence;
    }
    set confidence(newVal) {
        this._dict.Confidence = newVal;
    }
    get geometry() {
        return this._geometry;
    }
    get text() {
        return this._dict.Text;
    }
    getText(opts) {
        if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        return this._dict.Text;
    }
    html(opts) {
        if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        return (0, base_1.escapeHtml)(this.text);
    }
    str() {
        return `Line\n==========\n${this._dict.Text}\nWords\n----------\n${this.listWords()
            .map((word) => `[${word.str()}]`)
            .join("")}`;
    }
}
exports.LineGeneric = LineGeneric;
class SelectionElement extends base_1.ApiBlockWrapper {
    constructor(block) {
        super(block);
        this._geometry = new geometry_1.Geometry(block.Geometry, this);
    }
    get confidence() {
        return this._dict.Confidence;
    }
    set confidence(newVal) {
        this._dict.Confidence = newVal;
    }
    get geometry() {
        return this._geometry;
    }
    get isSelected() {
        if (this.selectionStatus === "SELECTED")
            return true;
        if (this.selectionStatus === "NOT_SELECTED")
            return false;
        throw new Error(`Block ${this.id} has unexpected SelectionStatus: ${this.selectionStatus}`);
    }
    get selectionStatus() {
        return this._dict.SelectionStatus;
    }
    set selectionStatus(newVal) {
        this._dict.SelectionStatus = newVal;
    }
    html(opts) {
        if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        return `<input type="checkbox" disabled ${this.selectionStatus === "SELECTED" ? "checked " : ""}/>`;
    }
    str() {
        return this.selectionStatus;
    }
    get text() {
        return this.selectionStatus;
    }
}
exports.SelectionElement = SelectionElement;
class Signature extends base_1.ApiBlockWrapper {
    constructor(block) {
        super(block);
        this._geometry = new geometry_1.Geometry(block.Geometry, this);
    }
    get confidence() {
        return this._dict.Confidence;
    }
    set confidence(newVal) {
        this._dict.Confidence = newVal;
    }
    get geometry() {
        return this._geometry;
    }
    html(opts) {
        if (!(0, base_1.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        return `<input class="signature" type="text" disabled value="[SIGNATURE]"/>`;
    }
    str() {
        return "/-------------\\\n| [SIGNATURE] |\n\\-------------/";
    }
    get text() {
        return "";
    }
}
exports.Signature = Signature;
//# sourceMappingURL=content.js.map