"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LayoutGeneric = exports.LayoutListGeneric = exports.LayoutTitleGeneric = exports.LayoutTextGeneric = exports.LayoutTableGeneric = exports.LayoutSectionHeaderGeneric = exports.LayoutPageNumberGeneric = exports.LayoutKeyValueGeneric = exports.LayoutHeaderGeneric = exports.LayoutFooterGeneric = exports.LayoutFigureGeneric = void 0;
const base_1 = require("./api-models/base");
const base_2 = require("./base");
const content_1 = require("./content");
const geometry_1 = require("./geometry");
class LayoutItemBaseGeneric extends base_2.PageHostedApiBlockWrapper {
    constructor(block, parentLayout) {
        super(block, parentLayout.parentPage);
        this._geometry = new geometry_1.Geometry(block.Geometry, this);
        this._parentLayout = parentLayout;
    }
    get confidence() {
        return this._dict.Confidence;
    }
    get geometry() {
        return this._geometry;
    }
    get nLayoutChildrenDirect() {
        return this.listLayoutChildren({ deep: false }).length;
    }
    get nLayoutChildrenTotal() {
        return this.listLayoutChildren({ deep: true }).length;
    }
    get parentLayout() {
        return this._parentLayout;
    }
    iterLayoutChildren(opts = {}) {
        return (0, base_2.getIterable)(() => this.listLayoutChildren(opts));
    }
    listLayoutChildren({ deep = false, includeBlockTypes = null, onUnexpectedBlockType = null, skipBlockTypes = null, } = {}) {
        let result = [];
        if (includeBlockTypes) {
            includeBlockTypes = (0, base_2.setIntersection)(base_1.LAYOUT_BLOCK_TYPES, (0, base_2.normalizeOptionalSet)(includeBlockTypes));
        }
        else {
            includeBlockTypes = base_1.LAYOUT_BLOCK_TYPES;
        }
        for (const childRaw of this.iterRelatedItemsByRelType("CHILD", {
            includeBlockTypes,
            onUnexpectedBlockType,
            skipBlockTypes,
        })) {
            const child = childRaw;
            result.push(child);
            if (deep)
                result = result.concat(child.listLayoutChildren({
                    deep,
                    includeBlockTypes,
                    onUnexpectedBlockType,
                    skipBlockTypes,
                }));
        }
        return result;
    }
}
class LayoutLineContainerItem extends (0, content_1.buildWithContent)({ contentTypes: ["LINE"] })(LayoutItemBaseGeneric) {
    getText(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        return this.listContent(opts)
            .map((c) => c.text)
            .join("\n");
    }
    iterTextLines() {
        return (0, base_2.getIterable)(() => this.listTextLines());
    }
    listTextLines() {
        return this.listContent();
    }
    get nTextLines() {
        return this.listTextLines().length;
    }
}
class LayoutFigureGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<div class="figure">${content}</div>`;
    }
    str() {
        return `#### Figure ####\n${this.text}${this.text ? "\n" : ""}################`;
    }
}
exports.LayoutFigureGeneric = LayoutFigureGeneric;
class LayoutFooterGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<div class="footer-el">${content}</div>`;
    }
    str() {
        return `---- Footer text ----\n${this.text}\n---------------------`;
    }
}
exports.LayoutFooterGeneric = LayoutFooterGeneric;
class LayoutHeaderGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<div class="header-el">${content}</div>`;
    }
    str() {
        return `---- Header text ----\n${this.text}\n---------------------`;
    }
}
exports.LayoutHeaderGeneric = LayoutHeaderGeneric;
class LayoutKeyValueGeneric extends LayoutLineContainerItem {
    _listContentIdsInFormField(field) {
        const keyContentIds = field.key.listWords().map((word) => word.id);
        const fieldVal = field.value;
        return fieldVal ? keyContentIds.concat(fieldVal.listContent().map((item) => item.id)) : keyContentIds;
    }
    _mapPageContentToFormFields() {
        const contentToFields = {};
        this.parentPage.form.listFields().forEach((field) => {
            this._listContentIdsInFormField(field).forEach((id) => {
                contentToFields[id] = field;
            });
        });
        return contentToFields;
    }
    iterFields() {
        return (0, base_2.getIterable)(() => this.listFields());
    }
    listFields() {
        const wordsToFields = this._mapPageContentToFormFields();
        const consumedIds = {};
        const result = [];
        for (const line of this.iterContent()) {
            for (const word of line.iterWords()) {
                if (word.id in consumedIds)
                    continue;
                const field = wordsToFields[word.id];
                if (field) {
                    result.push(field);
                    this._listContentIdsInFormField(field).forEach((id) => {
                        consumedIds[id] = true;
                    });
                }
            }
        }
        return result;
    }
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const wordsToFields = this._mapPageContentToFormFields();
        const consumedIds = {};
        const lineReprs = [];
        for (const line of this.iterContent(opts)) {
            const wordReprs = [];
            for (const word of line.iterWords(opts)) {
                const wordId = word.id;
                if (wordId in consumedIds)
                    continue;
                if (wordId in wordsToFields) {
                    const field = wordsToFields[wordId];
                    wordReprs.push(field.html(opts));
                    this._listContentIdsInFormField(field).forEach((id) => {
                        consumedIds[id] = true;
                    });
                }
                else {
                    wordReprs.push(word.html(opts));
                }
            }
            lineReprs.push(wordReprs.join(" "));
        }
        const lineReprTexts = lineReprs.filter((rep) => rep).join("\n");
        const content = lineReprTexts ? `\n${(0, base_2.indent)(lineReprTexts)}\n` : "";
        return `<div class="key-value">${content}</div>`;
    }
    str() {
        return `---- Key-value ----\n${this.text}\n-------------------`;
    }
}
exports.LayoutKeyValueGeneric = LayoutKeyValueGeneric;
class LayoutPageNumberGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<div class="page-num">${content}</div>`;
    }
    str() {
        return `---- Page number: ${this.text}`;
    }
}
exports.LayoutPageNumberGeneric = LayoutPageNumberGeneric;
class LayoutSectionHeaderGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<h2>${content}</h2>`;
    }
    str() {
        return `\n${this.text}\n${"-".repeat(this.text.length)}\n`;
    }
}
exports.LayoutSectionHeaderGeneric = LayoutSectionHeaderGeneric;
class LayoutTableGeneric extends LayoutLineContainerItem {
    _listContentIdsInTable(table) {
        const cellContentIds = table
            .listRows()
            .map((row) => row
            .listCells()
            .map((cell) => cell
            .listContent()
            .map((item) => item.id)
            .flat())
            .flat())
            .flat();
        const footerIds = table
            .listFooters()
            .map((foot) => foot
            .listWords()
            .map((word) => word.id)
            .flat())
            .flat();
        const titleIds = table
            .listTitles()
            .map((title) => title
            .listWords()
            .map((word) => word.id)
            .flat())
            .flat();
        return titleIds.concat(cellContentIds, footerIds);
    }
    _mapPageContentToTables() {
        const contentToTables = {};
        this.parentPage.listTables().forEach((table) => {
            this._listContentIdsInTable(table).forEach((id) => {
                contentToTables[id] = table;
            });
        });
        return contentToTables;
    }
    iterTables() {
        return (0, base_2.getIterable)(() => this.listTables());
    }
    listTables() {
        const wordsToTables = this._mapPageContentToTables();
        const consumedIds = {};
        const result = [];
        for (const line of this.iterContent()) {
            for (const word of line.iterWords()) {
                if (word.id in consumedIds)
                    continue;
                const table = wordsToTables[word.id];
                if (table) {
                    result.push(table);
                    this._listContentIdsInTable(table).forEach((id) => {
                        consumedIds[id] = true;
                    });
                }
            }
        }
        return result;
    }
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const wordsToTables = this._mapPageContentToTables();
        const consumedIds = {};
        const lineReprs = [];
        for (const line of this.iterContent(opts)) {
            const wordReprs = [];
            for (const word of line.iterWords(opts)) {
                const wordId = word.id;
                if (wordId in consumedIds)
                    continue;
                if (wordId in wordsToTables) {
                    const table = wordsToTables[wordId];
                    wordReprs.push(table.html(opts));
                    this._listContentIdsInTable(table).forEach((id) => {
                        consumedIds[id] = true;
                    });
                }
                else {
                    wordReprs.push(word.html(opts));
                }
            }
            lineReprs.push(wordReprs.join(" "));
        }
        const lineReprsText = lineReprs.filter((rep) => rep).join("\n");
        const content = lineReprsText ? `\n${(0, base_2.indent)(lineReprsText)}\n` : "";
        return `<div class="table">${content}</div>`;
    }
    str() {
        return ["|==== Table (structure unknown) ====|", this.text, "|==================================|"].join("\n");
    }
}
exports.LayoutTableGeneric = LayoutTableGeneric;
class LayoutTextGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<p>${content}</p>`;
    }
    str() {
        return this.text;
    }
}
exports.LayoutTextGeneric = LayoutTextGeneric;
class LayoutTitleGeneric extends LayoutLineContainerItem {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const rawText = this.getText(opts);
        const content = rawText ? `\n${(0, base_2.indent)((0, base_2.escapeHtml)(rawText))}\n` : "";
        return `<h1>${content}</h1>`;
    }
    str() {
        return `\n\n${this.text}\n${"=".repeat(this.text.length)}\n`;
    }
}
exports.LayoutTitleGeneric = LayoutTitleGeneric;
class LayoutListGeneric extends (0, content_1.buildWithContent)({ contentTypes: ["LAYOUT_TEXT"] })(LayoutItemBaseGeneric) {
    html(opts) {
        if (!(0, base_2.doesFilterAllowBlockType)(opts, this.blockType))
            return "";
        const itemHtmls = this.listContent(opts)
            .map((item) => item.html(opts))
            .filter((s) => s)
            .map((s) => `<li>${s}</li>`);
        const content = itemHtmls.length ? `\n${(0, base_2.indent)(itemHtmls.join("\n"))}\n` : "";
        return `<ul>${content}</ul>`;
    }
    iterTextLines() {
        return (0, base_2.getIterable)(() => this.listTextLines());
    }
    listTextLines() {
        function extractLinesRecursive(items) {
            return items
                .map((item) => {
                if (item.blockType === "LINE")
                    return [item];
                else if (typeof item.listContent !== "undefined")
                    return extractLinesRecursive(item.listContent());
                else
                    return [];
            })
                .flat();
        }
        return extractLinesRecursive(this.listContent());
    }
    str() {
        return this.text;
    }
    get nTextLines() {
        return this.listTextLines().length;
    }
    get text() {
        return this.listContent()
            .map((item) => `  - ${(0, base_2.indent)(item.text, { character: " ", count: 4, skipFirstLine: true })}`)
            .join("\n");
    }
}
exports.LayoutListGeneric = LayoutListGeneric;
class LayoutGeneric {
    constructor(parentPage) {
        this._parentPage = parentPage;
        for (const block of (0, base_2._implIterRelatedBlocksByRelType)("CHILD", { includeBlockTypes: base_1.LAYOUT_BLOCK_TYPES, onMissingBlockId: "error" }, parentPage, parentPage)) {
            switch (block.BlockType) {
                case "LAYOUT_FIGURE":
                    new LayoutFigureGeneric(block, this);
                    break;
                case "LAYOUT_HEADER":
                    new LayoutHeaderGeneric(block, this);
                    break;
                case "LAYOUT_FOOTER":
                    new LayoutFooterGeneric(block, this);
                    break;
                case "LAYOUT_KEY_VALUE":
                    new LayoutKeyValueGeneric(block, this);
                    break;
                case "LAYOUT_LIST":
                    new LayoutListGeneric(block, this);
                    break;
                case "LAYOUT_PAGE_NUMBER":
                    new LayoutPageNumberGeneric(block, this);
                    break;
                case "LAYOUT_SECTION_HEADER":
                    new LayoutSectionHeaderGeneric(block, this);
                    break;
                case "LAYOUT_TABLE":
                    new LayoutTableGeneric(block, this);
                    break;
                case "LAYOUT_TEXT":
                    new LayoutTextGeneric(block, this);
                    break;
                case "LAYOUT_TITLE":
                    new LayoutTitleGeneric(block, this);
                    break;
            }
        }
    }
    get nItems() {
        return this.nItemsTotal;
    }
    get nItemsDirect() {
        return this.listItems({ deep: false }).length;
    }
    get nItemsTotal() {
        return this.listItems({ deep: true }).length;
    }
    get parentPage() {
        return this._parentPage;
    }
    get text() {
        return this.listItems({ deep: false })
            .map((item) => item.text)
            .join("\n\n");
    }
    html(opts) {
        return this.listItems(Object.assign({ deep: false }, opts))
            .map((item) => item.html(opts))
            .filter((s) => s)
            .join("\n");
    }
    iterItems({ deep = true, includeBlockTypes = null, onUnexpectedBlockType = null, skipBlockTypes = null, } = {}) {
        return (0, base_2.getIterable)(() => this.listItems({ deep, includeBlockTypes, onUnexpectedBlockType, skipBlockTypes }));
    }
    listItems({ deep = true, includeBlockTypes = null, onUnexpectedBlockType = null, skipBlockTypes = null, } = {}) {
        let normIncludeBlockTypes = includeBlockTypes;
        if (normIncludeBlockTypes) {
            normIncludeBlockTypes = (0, base_2.normalizeOptionalSet)(normIncludeBlockTypes);
            normIncludeBlockTypes = (0, base_2.setIntersection)(base_1.LAYOUT_BLOCK_TYPES, normIncludeBlockTypes);
        }
        else {
            normIncludeBlockTypes = base_1.LAYOUT_BLOCK_TYPES;
        }
        const visitedIds = new Set();
        const result = [];
        for (const item of this.parentPage.listRelatedItemsByRelType("CHILD", {
            includeBlockTypes: normIncludeBlockTypes,
            onUnexpectedBlockType,
            skipBlockTypes,
        })) {
            if (visitedIds.has(item.id))
                continue;
            result.push(item);
            if (!deep) {
                item
                    .listLayoutChildren({
                    deep: true,
                    includeBlockTypes,
                    onUnexpectedBlockType,
                    skipBlockTypes,
                })
                    .forEach((child) => visitedIds.add(child.id));
            }
        }
        return result;
    }
    str() {
        const content = this.listItems({ deep: false })
            .map((item) => item.str())
            .join("\n\n");
        return [
            "",
            "#### BEGIN PAGE LAYOUT #################",
            content,
            "#### END PAGE LAYOUT   #################",
            "",
        ].join("\n");
    }
}
exports.LayoutGeneric = LayoutGeneric;
//# sourceMappingURL=layout.js.map